/*
 * MACDRV Cocoa application class declaration
 *
 * Copyright 2011, 2012 Ken Thomases for CodeWeavers Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

#import <AppKit/AppKit.h>

#include "macdrv_cocoa.h"


@class WineEventQueue;
@class WineWindow;


@interface WineApplication : NSApplication <NSApplicationDelegate>
{
    CFRunLoopSourceRef requestSource;
    NSMutableArray* requests;
    dispatch_queue_t requestsManipQueue;

    CGFloat primaryScreenHeight;
    BOOL primaryScreenHeightValid;

    NSMutableArray* eventQueues;
    NSLock*         eventQueuesLock;

    NSTimeInterval eventTimeAdjustment;

    NSMutableArray* observations;
    NSMutableArray* keyWindows;
    NSMutableSet* triedWindows;
    unsigned long windowFocusSerial;

    NSEvent* lastFlagsChanged;
    BOOL keyboardIsISO;
    WineWindow* lastTargetWindow;

    NSMutableDictionary* originalModes;
    NSMutableArray* orderedWineWindows;

    NSArray*    cursorFrames;
    int         cursorFrame;
    NSTimer*    cursorTimer;
    BOOL        cursorHidden;

    BOOL clippingCursor;
    CGRect cursorClipRect;
    CFMachPortRef cursorClippingEventTap;
    NSMutableArray* warpRecords;
    CGPoint synthesizedLocation;
    NSTimeInterval lastSetCursorPositionTime;
    NSTimeInterval lastEventTapEventTime;

    NSMutableArray* orderedRunningApplications;

    NSImage* applicationIcon;
}

@property (readonly, nonatomic) NSTimeInterval eventTimeAdjustment;
@property (readonly, copy, nonatomic) NSEvent* lastFlagsChanged;
@property (nonatomic) BOOL keyboardIsISO;
@property (readonly, nonatomic) BOOL areDisplaysCaptured;
@property (readonly, nonatomic) NSArray* orderedWineWindows;

    - (void) transformProcessToForeground;

    - (void) flipRect:(NSRect*)rect;

    - (BOOL) registerEventQueue:(WineEventQueue*)queue;
    - (void) unregisterEventQueue:(WineEventQueue*)queue;

    - (void) computeEventTimeAdjustmentFromTicks:(unsigned long long)tickcount uptime:(uint64_t)uptime_ns;

    - (void) invalidateGotFocusEvents;
    - (void) windowGotFocus:(WineWindow*)window;

    - (void) wineWindow:(WineWindow*)window
                ordered:(NSWindowOrderingMode)order
             relativeTo:(WineWindow*)otherWindow;

    - (BOOL) waitUntilQueryDone:(int*)done timeout:(NSDate*)timeout;

    - (void) keyboardSelectionDidChange;

@end

void OnMainThreadAsync(dispatch_block_t block);
